// 模拟后端动态生成路由
import { defineFakeRoute } from "vite-plugin-fake-server/client";

/**
 * roles：页面级别权限，这里模拟二种 "admin"、"common"
 * admin：管理员角色
 * common：普通角色
 */
const permissionRouter = {
  path: "/permission",
  meta: {
    title: "权限管理",
    icon: "ep:lollipop",
    rank: 10
  },
  children: [
    {
      path: "/permission/page/index",
      name: "PermissionPage",
      meta: {
        title: "页面权限",
        roles: ["admin", "common"]
      }
    },
    {
      path: "/permission/button/index",
      name: "PermissionButton",
      meta: {
        title: "按钮权限",
        roles: ["admin", "common"],
        auths: [
          "permission:btn:add",
          "permission:btn:edit",
          "permission:btn:delete"
        ]
      }
    }
  ]
};

const kernelRouter = {
  path: "/kernel",
  meta: {
    title: "内核管理",
    icon: "ep:magic-stick",
    rank: 10
  },
  children: [
    {
      path: "/kernel/page/index",
      name: "KernelPage",
      meta: {
        title: "内核管理",
        roles: ["admin", "common"],
        auths: ["permission:kernel:upload", "permission:kernel:online-upgrade"]
      }
    },
    {
      path: "/kernel/page/webconsole",
      name: "WebConsolePage",
      meta: {
        title: "WebConsolePage",
        roles: ["admin", "common"],
        auths: ["permission:kernel:upload", "permission:kernel:online-upgrade"]
      }
    }
  ]
};

export default defineFakeRoute([
  {
    url: "/get-async-routes",
    method: "get",
    response: request => {
      console.log(
        request.headers.authorization === "Bearer eyJhbGciOiJIUzUxMiJ9.common"
      );
      if (
        request.headers.authorization === "Bearer eyJhbGciOiJIUzUxMiJ9.common"
      ) {
        kernelRouter.children[0].meta.auths = [
          "permission:kernel:online-upgrade"
        ];
      } else {
        kernelRouter.children[0].meta.auths = [
          "permission:kernel:upload",
          "permission:kernel:online-upgrade"
        ];
      }
      return {
        success: true,
        data: [permissionRouter, kernelRouter]
      };
    }
  }
]);
