package api

import (
	"myproject/model"
	"myproject/util"

	"github.com/gin-gonic/gin"
)

type Meta struct {
	Title string   `json:"title"`
	Icon  string   `json:"icon,omitempty"`
	Rank  int      `json:"rank,omitempty"`
	Roles []string `json:"roles,omitempty"`
	Auths []string `json:"auths,omitempty"`
}

type AsyncRoute struct {
	Path     string        `json:"path"`
	Name     string        `json:"name"`
	Meta     *Meta         `json:"meta"`
	Children []*AsyncRoute `json:"children,omitempty"`
}

type AsyncRouteResult struct {
	Success bool          `json:"success"`
	Data    []*AsyncRoute `json:"data"`
}

var routes2 = `
{
  "success": true,
  "data": [
    {
      "path": "/permission",
      "meta": {
        "title": "权限管理",
        "icon": "ep:lollipop",
        "rank": 10
      },
      "children": [
        {
          "path": "/permission/page/index",
          "name": "PermissionPage",
          "meta": {
            "title": "页面权限",
            "roles": [
              "admin",
              "common"
            ]
          }
        },
        {
          "path": "/permission/button/index",
          "name": "PermissionButton",
          "meta": {
            "title": "按钮权限",
            "roles": [
              "admin",
              "common"
            ],
            "auths": [
              "permission:btn:add",
              "permission:btn:edit",
              "permission:btn:delete"
            ]
          }
        }
      ]
    },
    {
      "path": "/kernel",
      "meta": {
        "title": "内核管理",
        "icon": "ep:magic-stick",
        "rank": 10
      },
      "children": [
        {
          "path": "/kernel/page/index",
          "name": "KernelPage",
          "meta": {
            "title": "内核管理",
            "roles": [
              "admin",
              "common"
            ],
            "auths": [
              "permission:kernel:upload",
              "permission:kernel:online-upgrade"
            ]
          }
        },
        {
          "path": "/kernel/page/webconsole",
          "name": "WebConsolePage",
          "meta": {
            "title": "WebConsolePage",
            "roles": [
              "admin",
              "common"
            ],
            "auths": [
              "permission:kernel:upload",
              "permission:kernel:online-upgrade"
            ]
          }
        }
      ]
    }
  ]
}`

func AsyncRoutes(c *gin.Context) {
	// get user from context
	var user model.User = c.MustGet("user").(model.User)
	var auths []string
	for _, rn := range user.RoleNames {
		var roles []model.Role
		util.DB.Preload("Permissions").Where(model.Role{Name: rn}).Find(&roles)
		for _, r := range roles {
			for _, p := range r.Permissions {
				auths = append(auths, p.Name)
			}
		}
	}
	var routes []*AsyncRoute
	// routes = append(routes, &AsyncRoute{
	// 	Path: "/permission",
	// 	Meta: &Meta{
	// 		Title: "权限管理",
	// 		Icon:  "ep:lollipop",
	// 		Rank:  10,
	// 	},
	// 	Children: []*AsyncRoute{
	// 		{
	// 			Name: "PermissionPage",
	// 			Path: "/permission/page/index",
	// 			Meta: &Meta{
	// 				Title: "页面权限",
	// 				Roles: []string{"admin", "common"},
	// 			},
	// 		},
	// 		{
	// 			Name: "PermissionButton",
	// 			Path: "/permission/button/index",
	// 			Meta: &Meta{
	// 				Title: "按钮权限",
	// 				Roles: []string{"admin", "common"},
	// 				Auths: auths,
	// 			},
	// 		},
	// 	}})
	routes = append(routes, &AsyncRoute{
		Path: "/kernel",
		Meta: &Meta{
			Title: "内核管理",
			Icon:  "ep:magic-stick",
			Rank:  10,
		},
		Children: []*AsyncRoute{
			{
				Name: "KernelPage",
				Path: "/kernel/page/index",
				Meta: &Meta{
					Title: "内核管理",
					Roles: []string{"admin", "common"},
					Auths: auths,
				},
			},
			// {
			// 	Name: "WebConsolePage",
			// 	Path: "/kernel/page/webconsole",
			// 	Meta: &Meta{
			// 		Title: "WebConsolePage",
			// 		Roles: []string{"admin", "common"},
			// 		Auths: auths,
			// 	},
			// },
		}})
	routes = append(routes, &AsyncRoute{
		Path: "/users",
		Meta: &Meta{
			Title: "用户管理",
			Icon:  "ep:user",
			Rank:  10,
		},
		Children: []*AsyncRoute{
			{
				Name: "UserPage",
				Path: "/user/page/index",
				Meta: &Meta{
					Title: "用户管理",
					Roles: []string{"admin"},
					Auths: auths,
				},
			},
		}})

	routes = append(routes, &AsyncRoute{
		Path: "/tasks",
		Meta: &Meta{
			Title: "任务管理",
			Icon:  "ep:checked",
			Rank:  10,
		},
		Children: []*AsyncRoute{
			{
				Name: "TaskPage",
				Path: "/task/page/index",
				Meta: &Meta{
					Title: "任务管理",
					Roles: []string{"admin", "common"},
					Auths: auths,
				},
			},
		}})
	c.JSON(200, AsyncRouteResult{
		Success: true,
		Data:    routes,
	})
}
