package api

import (
	"fmt"
	"myproject/model"
	"myproject/util"
	"os/exec"
	"regexp"
	"strings"

	"github.com/gin-gonic/gin"
	go_version "github.com/hashicorp/go-version"
)

type KernelResult struct {
	Success bool         `json:"success"`
	Data    model.Kernel `json:"data"`
}

type KernelInput struct {
	Istls bool `json:"tls" form:"tls"`
}

type KernelListResult struct {
	Success bool           `json:"success"`
	Data    []model.Kernel `json:"data"`
}

type HostListResult struct {
	Success bool         `json:"success"`
	Data    []model.Host `json:"data"`
}

func UploadKernel(c *gin.Context) {
	var input KernelInput
	var kernel model.Kernel
	var lastedKernel model.Kernel
	if err := c.ShouldBindQuery(&input); err != nil {
		c.JSON(400, gin.H{"error": err.Error()})
		return
	}
	util.DB.Order("id desc").Limit(1).Find(&lastedKernel)
	if lastedKernel.Version == "" {
		lastedKernel.Version = "0.0.0"
	}
	v1, _ := go_version.NewVersion(lastedKernel.Version)
	file, _ := c.FormFile("file")
	file_name := file.Filename
	re := regexp.MustCompile(`^\d+(\.\d+)*\.zip$`)
	if !re.MatchString(file_name) {
		c.JSON(401, gin.H{"error": "文件名不合法"})
		return
	}
	version := strings.Split(file_name, ".zip")[0]
	v2, _ := go_version.NewVersion(version)
	if v1.GreaterThan(v2) {
		c.JSON(401, gin.H{"error": "版本号不合法"})
		return
	}
	outf := fmt.Sprintf("uploads/%s", file_name)
	if err := c.SaveUploadedFile(file, outf); err != nil {
		c.JSON(401, gin.H{"error": err.Error()})
	} else {
		// 解压zip outf
		cmd := exec.Command("bash", "-c", fmt.Sprintf(
			"unzip %s;mv -f %s/* ./;rm -f %s.zip;rm -rf %s", file_name, version, version, version,
		))
		cmd.Dir = "uploads"
		cmd.Run()
		cmd = exec.Command("bash", "-c",
			`dpkg-scanpackages ./ /dev/null | gzip -9c > ./Packages.gz`)
		cmd.Dir = "uploads"
		cmd.Run()
		kernel = model.Kernel{Version: version,
			Lts:    input.Istls,
			Tested: true,
		}
		util.DB.Save(&kernel)
		c.JSON(200, KernelResult{Success: true, Data: kernel})
	}
}

func GetKernelList(c *gin.Context) {
	var users []model.Kernel
	util.DB.Find(&users)
	c.JSON(200, KernelListResult{Success: true, Data: users})
}

func GetHostList(c *gin.Context) {
	var users []model.Host
	util.DB.Find(&users)
	c.JSON(200, HostListResult{Success: true, Data: users})
}
