package api

import (
	"myproject/model"
	"myproject/util"
	"time"

	"github.com/gin-gonic/gin"
)

type UserResult struct {
	Success bool       `json:"success"`
	Data    model.User `json:"data"`
}

type LoginInput struct {
	Username string `json:"username" binding:"required"`
	Password string `json:"password" binding:"required"`
}

type UserInput struct {
	Username string   `json:"username" binding:"required"`
	Password string   `json:"password" binding:"required"`
	Nickname string   `json:"nickname"`
	Roles    []string `json:"roles" binding:"required"`
}

type UserListResult struct {
	Success bool         `json:"success"`
	Data    []model.User `json:"data"`
}

func Login(c *gin.Context) {
	var input LoginInput
	var user model.User
	if err := c.ShouldBindJSON(&input); err != nil {
		c.JSON(400, gin.H{"error": err.Error()})
		return
	}
	if err := util.DB.Preload("Roles").Where("username = ? AND password = ?", input.Username, input.Password).First(&user).Error; err != nil {
		c.JSON(401, gin.H{"error": "用户名或密码错误"})
	} else {
		c.JSON(200, UserResult{Success: true, Data: user})
	}
}

func CreateOrUpdateUser(c *gin.Context) {
	var input UserInput
	if err := c.ShouldBindJSON(&input); err != nil {
		c.JSON(400, gin.H{"error": err.Error()})
		return
	}
	var rolems []model.Role
	for _, role := range input.Roles {
		var rolem model.Role
		util.DB.Where("name = ?", role).First(&rolem)
		if rolem.Name != "" {
			rolems = append(rolems, rolem)
		} else {
			c.JSON(401, gin.H{"error": "角色不存在"})
			return
		}
	}
	var user model.User
	util.DB.Preload("Roles").Where("username = ?", input.Username).First(&user)
	if user.Username != "" {
		// update roles
		util.DB.Model(&user).Association("Roles").Clear()
		util.DB.Model(&user).Association("Roles").Append(rolems)
		util.DB.Save(&user)
	} else {
		user := &model.User{
			Avatar:       "https://avatars.githubusercontent.com/u/44761321",
			Username:     input.Username,
			Password:     input.Password,
			Nickname:     input.Nickname,
			Roles:        rolems,
			ExpiresAt:    time.Now().Add(time.Hour * 24 * 365),
			AccessToken:  util.GenerateToken(input.Username, input.Password),
			RefreshToken: util.GenerateToken(input.Username, input.Password),
		}
		util.DB.Create(user)
		util.DB.Model(user).Association("Roles").Append(rolems)
	}

	c.JSON(201, UserResult{Success: true, Data: user})
}

func GetUserList(c *gin.Context) {
	var users []model.User
	util.DB.Preload("Roles").Find(&users)
	c.JSON(200, UserListResult{Success: true, Data: users})
}
