package router

import (
	"myproject/model"
	"myproject/router/api"
	"myproject/util"
	"strings"
	"time"

	"github.com/gin-gonic/gin"
)

func SetupRouter() *gin.Engine {
	r := gin.Default()

	// 鉴权
	r.Use(func(ctx *gin.Context) {

		// 免除路径SetupRouter
		if ctx.Request.URL.Path == "/login" || ctx.Request.URL.Path == "/sb-webconsole" || ctx.Request.URL.Path == "/refresh-token" || ctx.Request.URL.Path == "/kernels" {
			ctx.Next()
			return
		}

		if strings.Contains(ctx.Request.URL.Path, "/debs/") {
			ctx.Next()
			return
		}

		// 获取请求头中的token
		token := ctx.GetHeader("Authorization")

		// 判断token是否为空
		if token == "" {
			ctx.JSON(401, gin.H{
				"code": 401,
				"msg":  "token is invalid",
			})
			return
		} else {
			var user model.User
			util.DB.Preload("Roles").Where(model.User{AccessToken: strings.Split(token, " ")[1]}).First(&user)
			if user.AccessToken == "" {
				ctx.JSON(403, gin.H{
					"code": 403,
					"msg":  "token is invalid",
				})
				return
			} else {
				ctx.Set("user", user)
				// 鉴权成功，继续执行下一个中间件
				ctx.Next()
			}
		}
	})
	r.MaxMultipartMemory = 40 << 20 // 40MB
	// 用户接口
	r.POST("/login", api.Login)
	// r.GET("/webconsole", api.WebConsole)
	r.GET("/sb-webconsole", api.ServerSideWebConsole)
	r.POST("/users", api.CreateOrUpdateUser)
	r.GET("/users", api.GetUserList)
	r.GET("/get-async-routes", api.AsyncRoutes)
	r.POST("/refresh-token", func(ctx *gin.Context) {
		type Input struct {
			RefreshToken string `json:"refreshToken"`
		}
		type Output struct {
			Success bool `json:"success"`
			Data    struct {
				AccessToken  string    `json:"accessToken"`
				RefreshToken string    `json:"refreshToken"`
				ExpiresIn    time.Time `json:"expires"`
			} `json:"data"`
		}
		var input Input
		if err := ctx.ShouldBindJSON(&input); err != nil {
			ctx.JSON(400, gin.H{
				"code": 400,
				"msg":  "参数错误",
			})
			return
		}
		var user model.User
		util.DB.Where(model.User{RefreshToken: input.RefreshToken}).First(&user)
		if user.RefreshToken == "" {
			ctx.JSON(403, gin.H{
				"code": 403,
				"msg":  "token is invalid",
			})
			return
		} else {
			// user.AccessToken = util.GenerateToken(user.Username, user.Password)
			// user.RefreshToken = util.GenerateToken(user.Username, user.Password)
			// util.DB.Save(&user)
			output := Output{
				Success: true,
				Data: struct {
					AccessToken  string    `json:"accessToken"`
					RefreshToken string    `json:"refreshToken"`
					ExpiresIn    time.Time `json:"expires"`
				}{user.AccessToken, user.RefreshToken, time.Now().Add(time.Hour * 24)},
			}
			ctx.JSON(200, output)
		}
	})
	r.GET("/tasks", api.GetTaskList)
	r.POST("/tasks", api.NewTask)
	r.GET("/hosts", api.GetHostList)
	r.GET("/kernels", api.GetKernelList)
	r.POST("/kernels", api.UploadKernel)
	r.Static("/debs", "./uploads")
	return r
}
