package util

import (
	"encoding/json"
	"io"
	"myproject/agentpb"
	"myproject/model"
	"net"
	"strings"
	"time"

	"github.com/hashicorp/go-version"
	"github.com/matishsiao/goInfo"
	"github.com/sirupsen/logrus"
	"github.com/zcalusic/sysinfo"
	"google.golang.org/grpc"
	"google.golang.org/grpc/peer"
)

type agentServer struct {
	agentpb.UnimplementedFeedsServer
}

func (s *agentServer) Broadcast(stream agentpb.Feeds_BroadcastServer) error {
	for {
		p, _ := peer.FromContext(stream.Context())
		// logrus.Info(p.Addr)
		msg, err := stream.Recv()
		if err == io.EOF {
			return nil
		}
		if err != nil {
			var host *model.Host
			ip := strings.Split(p.Addr.String(), ":")[0]
			DB.Where("ip = ?", ip).First(&host)
			if host != nil {
				host.Status = model.Status_DOWN
				host.LastUpdateTime = time.Now()
				DB.Save(host)
			}
			return err
		}

		switch {
		case msg.GetCmd() == agentpb.UP:
			var args agentpb.UPcmdArgs
			_ = json.Unmarshal([]byte(msg.GetArgs()), &args)
			ip := strings.Split(p.Addr.String(), ":")[0]
			var lastedKernel model.Kernel
			DB.Order("id desc").First(&lastedKernel)
			if lastedKernel.Version == "" {
				lastedKernel.Version = "0.0.0"
			}
			v1, _ := version.NewVersion(args.KernelVersion)
			v2, _ := version.NewVersion(lastedKernel.Version)
			var host *model.Host
			var ck string
			DB.Where("ip = ?", ip).First(&host)
			if host.CandidateKernel != "" {
				ck = host.CandidateKernel
			} else {
				ck = args.KernelVersion
			}
			v3, _ := version.NewVersion(ck)
			host = &model.Host{
				Status:          model.StatusEnum(msg.GetStatus()),
				Hostname:        args.Hostname,
				Ip:              ip,
				Series:          args.Series,
				Arch:            args.Arch,
				KernelVersion:   args.KernelVersion,
				LastUpdateTime:  time.Now(),
				CandidateKernel: ck,
			}
			DB.Save(host)
			var tasks []model.Task
			DB.Where("host_ip = ? AND status = ?", ip, model.TaskStatus_Waiting).Find(&tasks)
			logrus.Infof("tasks for ip: %s, tasks: %v", ip, tasks)
			for _, task := range tasks {
				task.Status = model.TaskStatus_Dispacthed
			}
			if len(tasks) > 0 && v1.LessThan(v2) {
				DB.Save(&tasks)
				args1 := &agentpb.TaskCmdArgs{
					Task: tasks,
				}
				args1b, _ := json.Marshal(args1)
				stream.Send(&agentpb.AgentCommand{
					Cmd:     agentpb.Task,
					Args:    string(args1b),
					Status:  model.Status_UPGRADE,
					IsError: false,
				})
			}
			if v3.LessThan(v2) {
				host.CandidateKernel = lastedKernel.Version
				DB.Save(host)
				var tasks []model.Task
				tasks = append(tasks, model.Task{
					Status: model.TaskStatus_Dispacthed,
					Args:   lastedKernel.Version,
					Type:   model.UpgradeKernel,
					HostIp: ip,
				})
				DB.Save(tasks)
				args1 := &agentpb.TaskCmdArgs{
					Task: tasks,
				}
				args1b, _ := json.Marshal(args1)
				stream.Send(&agentpb.AgentCommand{
					Cmd:     agentpb.Task,
					Args:    string(args1b),
					Status:  model.Status_UPGRADE,
					IsError: false,
				})
			}
		case msg.GetCmd() == agentpb.TaskResult:
			var args agentpb.TaskResultCmdArgs
			_ = json.Unmarshal([]byte(msg.GetArgs()), &args)
			var task model.Task
			DB.Where("id = ?", args.Task.ID).First(&task)
			task.Status = args.Task.Status
			DB.Save(&task)
		}

		// feed := "New Feed Recieved: " + strconv.Itoa(int(msg.GetCmd())) + " addr: " + p.Addr.String()
		// go func() {
		// 	for {
		// 		stream.Send(&agentpb.AgentCommand{
		// 			Cmd:     agentpb.UP,
		// 			Status:  "OK",
		// 			IsError: false,
		// 		})
		// 		time.Sleep(time.Second * 2)
		// 	}
		// }()

		// logrus.Info(feed)
	}
}

func AgentRegist() {
	lis, err := net.Listen("tcp", ":50051")
	if err != nil {
		logrus.Fatalf("could not listen on port: %v", err)
	}
	s := grpc.NewServer()
	agentpb.RegisterFeedsServer(s, &agentServer{})
	if err := s.Serve(lis); err != nil {
		logrus.Fatalf("could not start the server: %v", err)
	}
}

type AgentInfo struct {
	goInfo.GoInfoObject
	AgentID string `json:"agent_id"`
	Series  string `json:"series"`
	Ip      string `json:"ip"`
}

func GetAgentInfo() (*AgentInfo, error) {
	gi, err := goInfo.GetInfo()
	var si sysinfo.SysInfo
	si.GetSysInfo()
	if err != nil {
		return nil, err
	}
	// determing the ip address
	// addrs, err := net.InterfaceAddrs()
	// if err != nil {
	// 	return nil, err
	// }
	// var ip string
	// for _, addr := range addrs {
	// 	if ipnet, ok := addr.(*net.IPNet); ok && !ipnet.IP.IsLoopback() {
	// 		if ipnet.IP.To4() != nil {
	// 			// is default route
	// 			if
	// 			ip = ipnet.IP.String()
	// 		}
	// 	}

	return &AgentInfo{
		GoInfoObject: gi,
		AgentID:      "agent_id",
		Series:       si.OS.Version,
	}, nil
}
